/********************************************************************\

  Name:         adccalib.c
  Created by:   Stefan Ritt

  Contents:     Example analyzer module for ADC calibration. Looks
                for ADC0 bank, subtracts pedestals and applies gain
                calibration. The resulting values are appended to 
                the event as an CADC bank ("calibrated ADC"). The
                pedestal values and software gains are stored in
                adccalib_param structure which was defined in the ODB
                and transferred to experim.h.

  $Log: adccalib.c,v $
  Revision 1.2  1998/10/12 12:18:58  midas
  Added Log tag in header


\********************************************************************/
                                                    
/*-- Include files -------------------------------------------------*/

/* standard includes */
#include <stdio.h>
#include <time.h>

/* midas includes */
#include "midas.h"
#include "experim.h"
#include "analyzer.h"

/* cernlib includes */
#ifdef OS_WINNT
#define VISUAL_CPLUSPLUS
#endif

#ifdef __linux__
#define f2cFortran
#endif

#include <cfortran.h>
#include <hbook.h>

/*-- Parameters ----------------------------------------------------*/

ADC_CALIBRATION_PARAM adccalib_param;
extern EXP_PARAM      exp_param;
extern RUNINFO        runinfo;

/*-- Module declaration --------------------------------------------*/

INT adc_calib(EVENT_HEADER*,void*);
INT adc_calib_init(void);
INT adc_calib_bor(INT run_number);
INT adc_calib_eor(INT run_number);

ADC_CALIBRATION_PARAM_STR(adc_calibration_param_str);

ANA_MODULE adc_calib_module = {
  "ADC calibration",             /* module name           */  
  "Stefan Ritt",                 /* author                */
  adc_calib,                     /* event routine         */
  adc_calib_bor,                 /* BOR routine           */
  adc_calib_eor,                 /* EOR routine           */
  adc_calib_init,                /* init routine          */
  NULL,                          /* exit routine          */
  &adccalib_param,               /* parameter structure   */
  sizeof(adccalib_param),        /* structure size        */
  adc_calibration_param_str,     /* initial parameters    */
};

/*-- init routine --------------------------------------------------*/

char *adc_tag[] = {
  "WP1_1L", /* ADC1, slot 18 */
  "WP1_2L",
  "WP1_3L",
  "WP1_4L",
  "WP1_1R",
  "WP1_2R",
  "WP1_3R",
  "WP1_4R",
  "WP2_1L",
  "WP2_2L",
  "WP2_3L",
  "WP2_4L",

  "bad",    /* ADC2, slot 19 */
  "WP2_1R",
  "WP2_2R",
  "WP2_3R",
  "WP2_4R",
  "ST2_1",
  "ST2_2",
  "ST2_3",
  "ST2_4",
  "ST2_5",
  "ST2_6",
  "ST2_7",

  "ST2_8",  /* ADC3, slot 20 */
  "ST2_9",
  "ST2_10",
  "ST2_11",
  "ST2_12",
  "ST2_13",
  "ST2_14",
  "ST2_15",
  "spare",
  "spare",
  "spare",
  "spare",

  "ST1_1",  /* ADC4, slot 21 */
  "ST1_2",
  "ST1_3",
  "ST1_4",
  "ST1_5",
  "ST1_6",
  "ST1_7",
  "ST1_8",
  "ST1_9",
  "ST1_10",
  "ST1_11",
  "ST1_12",

  "S0",  /* ADC5, slot 22 */
  "S1",
  "S2",
  "Spare1",
  "Spare2",
  "Spare3",
};

INT adc_calib_init(void)
{
int  i;
char str[80];

  /* book CADC histos */
  for (i=0; i<N_ADC; i++)  
    {
    strcpy(str, "ADC ");
    strcat(str, adc_tag[i]);
    HBOOK1(ADCCALIB_ID_BASE+i, str, 1025, -50.f, 2000.f, 0.f); 
    }

  return SUCCESS;
}

/*-- BOR routine ---------------------------------------------------*/

INT adc_calib_bor(INT run_number)
{
  return SUCCESS;
}

/*-- eor routine ---------------------------------------------------*/

INT adc_calib_eor(INT run_number)
{
  return SUCCESS;
}

/*-- event routine -------------------------------------------------*/

INT adc_calib(EVENT_HEADER *pheader, void *pevent)
{
INT    i, n_adc;
WORD   *pdata;
float  *cadc;

  /* look for ADC0 bank, return if not present */
  n_adc = bk_locate(pevent, "ADC0", &pdata);
  if (n_adc == 0 || n_adc > N_ADC)
    return ANA_CONTINUE;

  /* create calibrated ADC bank */
  bk_create(pevent, "CADC", TID_FLOAT, &cadc);

  /* zero cadc bank */
  for (i=0 ; i<N_ADC ; i++)
    cadc[i] = 0.f;

  /* subtract pedestal */
  for (i=0 ; i<n_adc ; i++)
    cadc[i] = (float) ((double)pdata[i] - adccalib_param.pedestal[i]);

  /* apply software gain calibration */
  for (i=0 ; i<n_adc ; i++)
    cadc[i] *= adccalib_param.software_gain[i];

  /* fill ADC histos if above threshold */
  for (i=0 ; i<n_adc ; i++)
    if (cadc[i] > (float) adccalib_param.histo_threshold)
      HF1(ADCCALIB_ID_BASE+i, cadc[i], 1.f);

  /* close calculated bank */
  bk_close(pevent, cadc+n_adc);

  return SUCCESS;
}
